@@ -1,13 +1,13 @@
 #include "common.h"
 #include "dbaccess.h"
 
-int readdb(char iface[32], char dirname[512])
+int readdb(const char *iface, const char *dirname)
 {
 	FILE *db;
 	char file[512], backup[512];
-	int newdb=0, i;
+	int newdb=0;
 
-	snprintf(file, 512, "%s/%s", dirname, iface);	
+	snprintf(file, 512, "%s/%s", dirname, iface);
 	snprintf(backup, 512, "%s/.%s", dirname, iface);
 
 	if ((db=fopen(file,"r"))!=NULL) {
@@ -55,7 +55,7 @@ int readdb(char iface[32], char dirname[512])
 						if (noexit) {
 							return -1;
 						} else {
-							exit(1);
+							exit(EXIT_FAILURE);
 						}
 					} else {
 						if (debug) {
@@ -72,7 +72,7 @@ int readdb(char iface[32], char dirname[512])
 							if (noexit) {
 								return -1;
 							} else {
-								exit(1);
+								exit(EXIT_FAILURE);
 							}
 						}
 					}
@@ -84,7 +84,7 @@ int readdb(char iface[32], char dirname[512])
 					if (noexit) {
 						return -1;
 					} else {
-						exit(1);
+						exit(EXIT_FAILURE);
 					}
 				}
 			}
@@ -97,7 +97,7 @@ int readdb(char iface[32], char dirname[512])
 			if (noexit) {
 				return -1;
 			} else {
-				exit(1);
+				exit(EXIT_FAILURE);
 			}		
 		}
 
@@ -122,65 +122,89 @@ int readdb(char iface[32], char dirname[512])
 		printe(PT_Error);
 		
 		newdb=1;
-		
-		/* set default values for a new database */
+		initdb();
 		strncpy(data.interface, iface, 32);
 		strncpy(data.nick, data.interface, 32);
-		data.version=DBVERSION;
-		data.active=1;
-		data.totalrx=0;
-		data.totaltx=0;
-		data.currx=0;
-		data.curtx=0;
-		data.totalrxk=0;
-		data.totaltxk=0;
-		data.lastupdated=time(NULL);
-		data.created=time(NULL);
-		
-		/* days */
-		for (i=0;i<=29;i++) {
-			data.day[i].rx=0;
-			data.day[i].tx=0;
-			data.day[i].rxk=0;
-			data.day[i].txk=0;
-			data.day[i].date=0;
-			data.day[i].used=0;
-		}
-		
-		/* months */
-		for (i=0;i<=11;i++) {
-			data.month[i].rx=0;
-			data.month[i].tx=0;
-			data.month[i].rxk=0;
-			data.month[i].txk=0;
-			data.month[i].month=0;
-			data.month[i].used=0;
-		}
-		
-		/* top10 */
-		for (i=0;i<=9;i++) {
-			data.top10[i].rx=0;
-			data.top10[i].tx=0;
-			data.top10[i].rxk=0;
-			data.top10[i].txk=0;
-			data.top10[i].date=0;
-			data.top10[i].used=0;
-		}
-		
-		/* hours */
-		for (i=0;i<=23;i++) {
-			data.hour[i].rx=0;
-			data.hour[i].tx=0;
-			data.hour[i].date=0;
-		}
-		data.day[0].used=data.month[0].used=1;
-		data.day[0].date=data.month[0].month=time(NULL);
-		data.btime=FP32;
 	}
 	return newdb;
 }
 
-int writedb(char iface[32], char dirname[512], int newdb)
+void initdb(void)
+{
+	int i;
+	time_t current;
+	struct tm *d;
+	
+	current=time(NULL);
+	d=localtime(&current);
+
+	/* set default values for a new database */
+	data.version=DBVERSION;
+	data.active=1;
+	data.totalrx=0;
+	data.totaltx=0;
+	data.currx=0;
+	data.curtx=0;
+	data.totalrxk=0;
+	data.totaltxk=0;
+	data.lastupdated=current;
+	data.created=current;
+
+	/* days */
+	for (i=0;i<=29;i++) {
+		data.day[i].rx=0;
+		data.day[i].tx=0;
+		data.day[i].rxk=0;
+		data.day[i].txk=0;
+		data.day[i].date=0;
+		data.day[i].used=0;
+	}
+	
+	/* months */
+	for (i=0;i<=11;i++) {
+		data.month[i].rx=0;
+		data.month[i].tx=0;
+		data.month[i].rxk=0;
+		data.month[i].txk=0;
+		data.month[i].month=0;
+		data.month[i].used=0;
+	}
+	
+	/* top10 */
+	for (i=0;i<=9;i++) {
+		data.top10[i].rx=0;
+		data.top10[i].tx=0;
+		data.top10[i].rxk=0;
+		data.top10[i].txk=0;
+		data.top10[i].date=0;
+		data.top10[i].used=0;
+	}
+
+	/* hours */
+	for (i=0;i<=23;i++) {
+		data.hour[i].rx=0;
+		data.hour[i].tx=0;
+		data.hour[i].date=0;
+	}
+	
+	data.day[0].used=data.month[0].used=1;
+	data.day[0].date=current;
+
+	/* calculate new date for current month if current day is less
+           than the set monthrotate value so that new databases begin
+           from the right month */
+	if (d->tm_mday < cfg.monthrotate) {
+		d->tm_mday=cfg.monthrotate;
+		d->tm_mon--;
+		data.month[0].month=mktime(d);
+	} else {
+		data.month[0].month=current;
+	}
+
+	data.btime=FP32;
+}
+
+int writedb(const char *iface, const char *dirname, int newdb)
 {
 	FILE *db;
 	char file[512], backup[512];
@@ -205,7 +229,11 @@ int writedb(char iface[32], char dirname[512], int newdb)
 			return 0;
 		}
 
-		data.lastupdated=time(NULL);
+		/* update timestamp when not merging */
+		if (newdb!=2) {
+			data.lastupdated=time(NULL);
+		}
+
 		if (fwrite(&data,sizeof(DATA),1,db)==0) {
 			snprintf(errorstring, 512, "Unable to write database \"%s\".", file);
 			printe(PT_Error);
@@ -230,7 +258,7 @@ int writedb(char iface[32], char dirname[512], int newdb)
 	return 1;
 }
 
-int backupdb(char *current, char *backup)
+int backupdb(const char *current, const char *backup)
 {
 	FILE *bf;
 	int c, b, bytes;
@@ -249,7 +277,7 @@ int backupdb(char *current, char *backup)
 	b = fileno(bf);
 
 	/* copy data */
-	while((bytes = read(c, buffer, sizeof(buffer))) > 0) {
+	while((bytes = (int)read(c, buffer, sizeof(buffer))) > 0) {
 		if (write(b, buffer, bytes) < 0) {
 			close(c);
 			fclose(bf);
@@ -270,7 +298,6 @@ int convertdb(FILE *db)
 	DATA12 data12;
 	time_t current;
 	struct tm *d;
-	int dmonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 	int month=0, day;
 	int tm_mday, tm_mon, tm_year;
 	int converted=0;
@@ -319,7 +346,8 @@ int convertdb(FILE *db)
 				data12.day[i].date=current-(i*86400);
 				data12.day[i].used=1;
 			} else {
-				data12.day[i].rx=data12.day[i].tx=0;
+				data12.day[i].rx=0;
+				data12.day[i].tx=0;
 				data12.day[i].used=0;
 			}			
 		}
@@ -336,10 +364,11 @@ int convertdb(FILE *db)
 				mod=(d->tm_mon-i-1)%12;
 				if (mod<0)
 					mod=12+mod;
-				days+=dmonth[mod];
+				days+=dmonth(mod);
 				data12.month[i].used=1;
 			} else {
-				data12.month[i].rx=data12.month[i].tx=0;
+				data12.month[i].rx=0;
+				data12.month[i].tx=0;
 				data12.month[i].used=0;
 			}			
 		}		
@@ -400,7 +429,8 @@ int convertdb(FILE *db)
 
 			} else {
 				data12.top10[i].used=0;
-				data12.top10[i].rx=data12.top10[i].tx=0;
+				data12.top10[i].rx=0;
+				data12.top10[i].tx=0;
 			}
 			
 			
@@ -576,6 +606,37 @@ int lockdb(int fd, int dbwrite)
 	return 1;
 }
 
+int checkdb(const char *iface, const char *dirname)
+{
+	char file[512];
+	struct statvfs buf;
+	
+	snprintf(file, 512, "%s/%s", dirname, iface);
+	
+	if (statvfs(file, &buf)==0) {
+		return 1; /* file exists */	
+	} else {
+		return 0; /* no file or some other error */
+	}
+}
+
+int removedb(const char *iface, const char *dirname)
+{
+	char file[512];
+	
+	/* remove backup first */
+	snprintf(file, 512, "%s/.%s", dirname, iface);
+	unlink(file);
+	
+	snprintf(file, 512, "%s/%s", dirname, iface);
+	if (unlink(file)!=0) {
+		perror("unlink");
+		return 0;
+	}
+	
+	return 1;
+}
+
 void cleanhours(void)
 {
 	int i, day, hour;
@@ -591,7 +652,7 @@ void cleanhours(void)
 			data.hour[i].tx=0;
 			data.hour[i].date=0;
 			if (debug) {
-				printf("db: Hour %d (%d) cleaned.\n", i, (int)data.hour[i].date);
+				printf("db: Hour %d (%u) cleaned.\n", i, (unsigned int)data.hour[i].date);
 			}
 		}
 	}
@@ -605,7 +666,7 @@ void cleanhours(void)
 			data.hour[hour].rx=0;
 			data.hour[hour].tx=0;
 			if (debug) {
-				printf("db: Current hour %d (%d) cleaned.\n", hour, (int)data.hour[hour].date);
+				printf("db: Current hour %d (%u) cleaned.\n", hour, (unsigned int)data.hour[hour].date);
 			}
 			data.hour[hour].date=current;
 	}
