@@ -3,7 +3,7 @@
 #include "misc.h"
 #include "traffic.h"
 
-void trafficmeter(char iface[32], int sampletime)
+void trafficmeter(char iface[], int sampletime)
 {
 	/* received bytes packets errs drop fifo frame compressed multicast */
 	/* transmitted bytes packets errs drop fifo colls carrier compressed */
@@ -15,13 +15,13 @@ void trafficmeter(char iface[32], int sampletime)
 	/* less than 2 seconds doesn't produce good results */
 	if (sampletime<2) {
 		printf("Error: Time for sampling too short.\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* read interface info and get values to the first list */
 	if (!getifinfo(iface)) {
 		printf("Error: Interface \"%s\" not available, exiting.\n", iface);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 	firstinfo.rx = ifinfo.rx;
 	firstinfo.tx = ifinfo.tx;
@@ -30,8 +30,8 @@ void trafficmeter(char iface[32], int sampletime)
 
 	/* wait sampletime and print some nice dots so that the user thinks
 	something is done :) */
-	sprintf(buffer,"Sampling %s (%d seconds average)",iface,sampletime);
-	printf("%s",buffer);
+	snprintf(buffer, 256, "Sampling %s (%d seconds average)", iface,sampletime);
+	printf("%s", buffer);
 	fflush(stdout);
 	sleep(sampletime/3);
 	printf(".");
@@ -55,24 +55,24 @@ void trafficmeter(char iface[32], int sampletime)
 	/* read those values again... */
 	if (!getifinfo(iface)) {
 		printf("Error: Interface \"%s\" not available, exiting.\n", iface);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* calculate traffic and packets seen between updates */
-	rx = countercalc(firstinfo.rx, ifinfo.rx);
-	tx = countercalc(firstinfo.tx, ifinfo.tx);
+	rx = rintf(countercalc(firstinfo.rx, ifinfo.rx)/(float)1024);
+	tx = rintf(countercalc(firstinfo.tx, ifinfo.tx)/(float)1024);
 	rxp = countercalc(firstinfo.rxp, ifinfo.rxp);
 	txp = countercalc(firstinfo.txp, ifinfo.txp);
 
 	/* show the difference in a readable format */
 	printf("%"PRIu64" packets sampled in %d seconds\n", rxp+txp, sampletime);
 	printf("Traffic average for %s\n", iface);
-	printf("\n      rx     %10.2f %s/s         %5"PRIu64" packets/s\n", rx/(float)sampletime/(float)1024, getunit(1), (uint64_t)(rxp/sampletime));
-	printf("      tx     %10.2f %s/s         %5"PRIu64" packets/s\n\n", tx/(float)sampletime/(float)1024, getunit(1), (uint64_t)(txp/sampletime));
+	printf("\n      rx     %s         %5"PRIu64" packets/s\n", getrate(0, rx, sampletime, 15), (uint64_t)(rxp/sampletime));
+	printf("      tx     %s         %5"PRIu64" packets/s\n\n", getrate(0, tx, sampletime, 15), (uint64_t)(txp/sampletime));
 
 }
 
-void livetrafficmeter(char iface[32])
+void livetrafficmeter(char iface[32], int mode)
 {
 	/* received bytes packets errs drop fifo frame compressed multicast */
 	/* transmitted bytes packets errs drop fifo colls carrier compressed */
@@ -80,25 +80,27 @@ void livetrafficmeter(char iface[32])
 	uint64_t rxtotal, txtotal, rxptotal, txptotal;
 	uint64_t rxpmin, txpmin, rxpmax, txpmax;
 	float rxmin, txmin, rxmax, txmax, rxc, txc;
-	int i, len;
-	char buffer[256];
+	int i, len=0;
+	char buffer[256], buffer2[256];
 	IFINFO previnfo;
 
 	printf("Monitoring %s...    (press CTRL-C to stop)\n\n", iface);
-	printf("   getting traffic...");
-	len=21; /* length of previous print */
-	fflush(stdout);
+	if (cfg.ostyle != 4) {
+		printf("   getting traffic...");
+		len=21; /* length of previous print */
+		fflush(stdout);
+	}
 
 	/* enable signal trap */
 	intsignal = 0;
 	if (signal(SIGINT, sighandler) == SIG_ERR) {
 		perror("signal");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* set some defaults */
 	rxtotal=txtotal=rxptotal=txptotal=rxpmax=txpmax=0;
-	rxpmin=txpmin=-1;
+	rxpmin=txpmin=FP64;
 	rxmax=txmax=0.0;
 	rxmin=txmin=-1.0;
 	
@@ -107,7 +109,7 @@ void livetrafficmeter(char iface[32])
 	/* read /proc/net/dev and get values to the first list */
 	if (!getifinfo(iface)) {
 		printf("Error: Interface \"%s\" not available, exiting.\n", iface);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* loop until user gets bored */
@@ -130,7 +132,7 @@ void livetrafficmeter(char iface[32])
 		/* read those values again... */
 		if (!getifinfo(iface)) {
 			printf("Error: Interface \"%s\" not available, exiting.\n", iface);
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 
 		/* calculate traffic and packets seen between updates */
@@ -145,8 +147,8 @@ void livetrafficmeter(char iface[32])
 		rxptotal += rxp;
 		txptotal += txp;
 
-		rxc = rx/(float)LIVETIME/(float)1024;
-		txc = tx/(float)LIVETIME/(float)1024;
+		rxc = rintf(rx/(float)1024);
+		txc = rintf(tx/(float)1024);
 		rxpc = rxp/LIVETIME;
 		txpc = txp/LIVETIME;		
 
@@ -164,10 +166,10 @@ void livetrafficmeter(char iface[32])
 			txmax = txc;
 		}
 		
-		if ((rxpmin==-1) || (rxpmin>rxpc)) {
+		if ((rxpmin==FP64) || (rxpmin>rxpc)) {
 			rxpmin = rxpc;
 		}
-		if ((txpmin==-1) || (txpmin>txpc)) {
+		if ((txpmin==FP64) || (txpmin>txpc)) {
 			txpmin = txpc;
 		}
 		if (rxpmax<rxpc) {
@@ -178,9 +180,26 @@ void livetrafficmeter(char iface[32])
 		}
 
 		/* show the difference in a readable format */
-		snprintf(buffer, 256, "   rx: %10.2f %s/s %5"PRIu64" p/s          tx: %10.2f %s/s %5"PRIu64" p/s", rxc, getunit(1), (uint64_t)rxpc, txc, getunit(1), (uint64_t)txpc);
+		if (cfg.ostyle != 0) {
+			if (mode == 0) {
+				snprintf(buffer, 128, "   rx: %s %5"PRIu64" p/s", getrate(0, rxc, LIVETIME, 15), (uint64_t)rxpc);
+				snprintf(buffer2, 128, "          tx: %s %5"PRIu64" p/s", getrate(0, txc, LIVETIME, 15), (uint64_t)txpc);
+			} else {
+				snprintf(buffer, 128, "   rx: %s   %s", getrate(0, rxc, LIVETIME, 13), getvalue(0, rintf(rxtotal/(float)1024), 1, 1));
+				snprintf(buffer2, 128, "             tx: %s   %s", getrate(0, txc, LIVETIME, 13), getvalue(0, rintf(txtotal/(float)1024), 1, 1));
+			}
+		} else {
+			if (mode == 0) {
+				snprintf(buffer, 128, "   rx: %s %3"PRIu64" p/s", getrate(0, rxc, LIVETIME, 12), (uint64_t)rxpc);
+				snprintf(buffer2, 128, "      tx: %s %3"PRIu64" p/s", getrate(0, txc, LIVETIME, 12), (uint64_t)txpc);
+			} else {
+				snprintf(buffer, 128, "  rx: %s  %s", getrate(0, rxc, LIVETIME, 12), getvalue(0, rintf(rxtotal/(float)1024), 1, 1));
+				snprintf(buffer2, 128, "       tx: %s  %s", getrate(0, txc, LIVETIME, 12), getvalue(0, rintf(txtotal/(float)1024), 1, 1));
+			}
+		}
+		strncat(buffer, buffer2, 127);
 		
-		if (len>1) {
+		if (len>1 && cfg.ostyle!=4) {
 			if (debug) {
 				printf("\nlinelen: %d\n", len);
 			} else {
@@ -190,8 +209,12 @@ void livetrafficmeter(char iface[32])
 				fflush(stdout);
 			}
 		}
-		printf("%s", buffer);
-		fflush(stdout);
+		if (cfg.ostyle!=4) {
+			printf("%s", buffer);
+			fflush(stdout);
+		} else {
+			printf("%s\n", buffer);
+		}
 		len=strlen(buffer);
 	
 	}
@@ -205,35 +228,26 @@ void livetrafficmeter(char iface[32])
 
 		printf("\n %s  /  traffic statistics\n\n", iface);
 
-		printf("                             rx       |       tx\n");
-		printf("--------------------------------------+----------------------------------------\n");
-		printf("  bytes                %s", getvalue(0, rxtotal/1024, 9, -1));
-		printf("  | %s", getvalue(0, txtotal/1024, 9, -1));
+		printf("                           rx         |       tx\n");
+		printf("--------------------------------------+------------------\n");
+		printf("  bytes                %s", getvalue(0, rintf(rxtotal/(float)1024), 13, 1));
+		printf("  |   %s", getvalue(0, rintf(txtotal/(float)1024), 13, 1));
 		printf("\n");
-		printf("--------------------------------------+----------------------------------------\n");
-		printf("          max          ");
-		showspeed(rxmax, 7);
-		printf("  | ");
-		showspeed(txmax, 7);
-		printf("\n");
-		printf("      average          ");
-		showspeed(rxtotal/(float)timespent/(float)1024, 7);
-		printf("  | ");
-		showspeed(txtotal/(float)timespent/(float)1024, 7);
-		printf("\n");
-		printf("          min          ");
-		showspeed(rxmin, 7);
-		printf("  | ");
-		showspeed(txmin, 7);
-		printf("\n");
-		printf("--------------------------------------+----------------------------------------\n");
-		printf("  packets               %12"PRIu64"  |  %12"PRIu64"\n", rxptotal, txptotal);
-		printf("--------------------------------------+----------------------------------------\n");
-		printf("          max          %9"PRIu64" p/s  | %9"PRIu64" p/s\n", rxpmax, txpmax);
-		printf("      average          %9"PRIu64" p/s  | %9"PRIu64" p/s\n", rxptotal/timespent, txptotal/timespent);
-		printf("          min          %9"PRIu64" p/s  | %9"PRIu64" p/s\n", rxpmin, txpmin);
-		printf("--------------------------------------+----------------------------------------\n");
-		
+		printf("--------------------------------------+------------------\n");
+		printf("          max          %s", getrate(0, rxmax, LIVETIME, 13));
+		printf("  |   %s\n", getrate(0, txmax, LIVETIME, 13));
+		printf("      average          %s", getrate(0, rintf(rxtotal/(float)1024), timespent, 13));
+		printf("  |   %s\n", getrate(0, rintf(txtotal/(float)1024), timespent, 13));
+		printf("          min          %s", getrate(0, rxmin, LIVETIME, 13));
+		printf("  |   %s\n", getrate(0, txmin, LIVETIME, 13));
+		printf("--------------------------------------+------------------\n");
+		printf("  packets               %12"PRIu64"  |    %12"PRIu64"\n", rxptotal, txptotal);
+		printf("--------------------------------------+------------------\n");
+		printf("          max          %9"PRIu64" p/s  |   %9"PRIu64" p/s\n", rxpmax, txpmax);
+		printf("      average          %9"PRIu64" p/s  |   %9"PRIu64" p/s\n", rxptotal/timespent, txptotal/timespent);
+		printf("          min          %9"PRIu64" p/s  |   %9"PRIu64" p/s\n", rxpmin, txpmin);
+		printf("--------------------------------------+------------------\n");
+
 		if (timespent<=60) {
 			printf("  time             %9"PRIu64" seconds\n", timespent);
 		} else {
@@ -244,24 +258,3 @@ void livetrafficmeter(char iface[32])
 	}
 	
 }
-
-void showspeed(float xfer, int len)
-{
-	if (cfg.unit>0) {
-		len++;
-	}
-
-#if !defined(__OpenBSD__)
-	if (xfer>=1000) {
-		printf("%'*.2f %s/s", len, xfer/(float)1024, getunit(2));
-	} else {
-		printf("%'*.2f %s/s", len, xfer, getunit(1));
-	}
-#else
-	if (xfer>=1000) {
-		printf("%*.2f %s/s", len, xfer/(float)1024, getunit(2));
-	} else {
-		printf("%*.2f KiB/s", len, xfer, getunit(1));
-	}
-#endif
-}
