@@ -1,5 +1,5 @@
 /*
-vnStat - Copyright (c) 2002-09 Teemu Toivola <tst@iki.fi>
+vnStat - Copyright (c) 2002-10 Teemu Toivola <tst@iki.fi>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ vnStat - Copyright (c) 2002-09 Teemu Toivola <tst@iki.fi>
 #include "dbxml.h"
 #include "dbshow.h"
 #include "dbaccess.h"
+#include "dbmerge.h"
 #include "misc.h"
 #include "cfg.h"
 #include "vnstat.h"
@@ -28,14 +29,14 @@ vnStat - Copyright (c) 2002-09 Teemu Toivola <tst@iki.fi>
 int main(int argc, char *argv[]) {
 
 	int i;
-	int currentarg, update=0, query=1, newdb=0, reset=0, sync=0;
+	int currentarg, update=0, query=1, newdb=0, reset=0, sync=0, merged=0, savemerged=0;
 	int active=-1, files=0, force=0, cleartop=0, rebuildtotal=0, traffic=0;
-	int livetraffic=0;
+	int livetraffic=0, defaultiface=1, delete=0, livemode=0;
 	char interface[32], dirname[512], nick[32];
-	char definterface[32], cfgfile[512], *ifacelist;
+	char definterface[32], cfgfile[512], *ifacelist=NULL;
 	time_t current;
-	DIR *dir;
-	struct dirent *di;
+	DIR *dir=NULL;
+	struct dirent *di=NULL;
 
 	noexit = 0; /* allow functions to exit in case of error */
 	debug = 0; /* debug disabled by default */
@@ -66,7 +67,13 @@ int main(int argc, char *argv[]) {
 		return 1;
 	}
 
-	setlocale(LC_ALL, cfg.locale);
+	if (cfg.locale[0]!='-' && strlen(cfg.locale)>0) {
+		setlocale(LC_ALL, cfg.locale);
+	} else {
+		if (getenv("LC_ALL")) {
+			setlocale(LC_ALL, getenv("LC_ALL"));
+		}
+	}
 	strncpy(interface, "default", 32);
 	strncpy(definterface, cfg.iface, 32);
 	strncpy(nick, "none", 32);
@@ -102,6 +109,8 @@ int main(int argc, char *argv[]) {
 			printf("         -w, --weeks           show weeks\n");
 			printf("         -t, --top10           show top10\n");
 			printf("         -s, --short           use short output\n");
+			printf("         -ru, --rateunit       swap configured rate unit\n");
+			printf("         --oneline             show simple parseable format\n");
 			printf("         --dumpdb              show database in parseable format\n");
 			printf("         --xml                 show database in xml format\n");
 			
@@ -112,8 +121,13 @@ int main(int argc, char *argv[]) {
 			printf("         -v,  --version        show version\n");
 			printf("         -tr, --traffic        calculate traffic\n");
 			printf("         -l,  --live           show transfer rate in real time\n");
+			printf("         --style               select output style (0-4)\n");
+			printf("         --delete              delete database and stop monitoring\n");
 			printf("         --iflist              show list of available interfaces\n");
-			printf("         --config              select used config file\n");
+			printf("         --dbdir               select database directory\n");
+			printf("         --locale              set locale\n");
+			printf("         --config              select config file\n");
+			printf("         --savemerged          save merged database to current directory\n");
 			printf("         --showconfig          dump config file with current settings\n");
 			printf("         --testkernel          check if the kernel is broken\n");
 			printf("         --longhelp            display this help\n\n");
@@ -138,6 +152,7 @@ int main(int argc, char *argv[]) {
 			printf("         -?,  --help           short help\n");
 			printf("         -v,  --version        show version\n");
 			printf("         -tr, --traffic        calculate traffic\n");
+			printf("         -ru, --rateunit       swap configured rate unit\n");
 			printf("         -l,  --live           show transfer rate in real time\n\n");
 
 			printf("See also \"--longhelp\" for complete options list and \"man vnstat\".\n");
@@ -147,6 +162,7 @@ int main(int argc, char *argv[]) {
 		} else if ((strcmp(argv[currentarg],"-i")==0) || (strcmp(argv[currentarg],"--iface")==0)) {
 			if (currentarg+1<argc) {
 				strncpy(interface, argv[currentarg+1], 32);
+				defaultiface = 0;
 				if (debug)
 					printf("Used interface: %s\n", interface);
 				currentarg++;
@@ -156,7 +172,7 @@ int main(int argc, char *argv[]) {
 				return 1;
 			}
 		} else if (strcmp(argv[currentarg],"--config")==0) {
-			/* config has already been parsed earlier so not but to do here*/
+			/* config has already been parsed earlier so nothing to do here */
 			currentarg++;
 			continue;
 		} else if ((strcmp(argv[currentarg],"--nick"))==0) {
@@ -170,6 +186,55 @@ int main(int argc, char *argv[]) {
 				printf("Error: Nick for --nick missing.\n");
 				return 1;
 			}
+		} else if ((strcmp(argv[currentarg],"--style"))==0) {
+			if (currentarg+1<argc && isdigit(argv[currentarg+1][0])) {
+				cfg.ostyle = atoi(argv[currentarg+1]);
+				if (cfg.ostyle > 4 || cfg.ostyle < 0) {
+					printf("Error: Invalid style parameter \"%d\" for --style.\n", cfg.ostyle);
+					printf(" Valid parameters:\n");
+					printf("    0 - a more narrow output\n");
+					printf("    1 - enable bar column if available\n");
+					printf("    2 - average traffic rate in summary and weekly outputs\n");
+					printf("    3 - average traffic rate in all outputs if available\n");
+					printf("    4 - disable terminal control characters in -l / --live\n");
+					return 1;
+				}
+				if (debug)
+					printf("Style changed: %d\n", cfg.ostyle);
+				currentarg++;
+				continue;
+			} else {
+				printf("Error: Style parameter for --style missing.\n");
+				printf(" Valid parameters:\n");
+				printf("    0 - a more narrow output\n");
+				printf("    1 - enable bar column if available\n");
+				printf("    2 - average traffic rate in summary and weekly outputs\n");
+				printf("    3 - average traffic rate in all outputs if available\n");
+				printf("    4 - disable terminal control characters in -l / --live\n");
+				return 1;
+			}
+		} else if ((strcmp(argv[currentarg],"--dbdir"))==0) {
+			if (currentarg+1<argc) {
+				strncpy(dirname, argv[currentarg+1], 512);
+				if (debug)
+					printf("DatabaseDir: \"%s\"\n", dirname);
+				currentarg++;
+				continue;
+			} else {
+				printf("Error: Directory for --dbdir missing.\n");
+				return 1;
+			}
+		} else if ((strcmp(argv[currentarg],"--locale"))==0) {
+			if (currentarg+1<argc) {
+				setlocale(LC_ALL, argv[currentarg+1]);
+				if (debug)
+					printf("Locale: \"%s\"\n", argv[currentarg+1]);
+				currentarg++;
+				continue;
+			} else {
+				printf("Error: Locale for --locale missing.\n");
+				return 1;
+			}
 		} else if ((strcmp(argv[currentarg],"-u")==0) || (strcmp(argv[currentarg],"--update")==0)) {
 			update=1;
 			query=0;
@@ -193,8 +258,31 @@ int main(int argc, char *argv[]) {
 			cfg.qmode=7;
 		} else if (strcmp(argv[currentarg],"--dumpdb")==0) {
 			cfg.qmode=4;
+		} else if (strcmp(argv[currentarg],"--oneline")==0) {
+			cfg.qmode=9;
 		} else if (strcmp(argv[currentarg],"--xml")==0) {
 			cfg.qmode=8;
+		} else if (strcmp(argv[currentarg],"--savemerged")==0) {
+			savemerged=1;
+		} else if ((strcmp(argv[currentarg],"-ru")==0) || (strcmp(argv[currentarg],"--rateunit"))==0) {
+			if (currentarg+1<argc && isdigit(argv[currentarg+1][0])) {
+				cfg.rateunit = atoi(argv[currentarg+1]);
+				if (cfg.rateunit > 1 || cfg.rateunit < 0) {
+					printf("Error: Invalid parameter \"%d\" for --rateunit.\n", cfg.rateunit);
+					printf(" Valid parameters:\n");
+					printf("    0 - bytes\n");
+					printf("    1 - bits\n");
+					return 1;
+				}
+				if (debug)
+					printf("Rateunit changed: %d\n", cfg.rateunit);
+				currentarg++;
+				continue;
+			} else {
+				cfg.rateunit = !cfg.rateunit;
+				if (debug)
+					printf("Rateunit changed: %d\n", cfg.rateunit);
+			}
 		} else if (strcmp(argv[currentarg],"--enable")==0) {
 			active=1;
 			query=0;
@@ -211,6 +299,17 @@ int main(int argc, char *argv[]) {
 			traffic=1;
 			query=0;
 		} else if ((strcmp(argv[currentarg],"-l")==0) || (strcmp(argv[currentarg],"--live")==0)) {
+			if (currentarg+1<argc && argv[currentarg+1][0]!='-') {
+				livemode = atoi(argv[currentarg+1]);
+				if (!isdigit(argv[currentarg+1][0]) || livemode > 1 || livemode < 0) {
+					printf("Error: Invalid mode parameter \"%s\" for -l / --live.\n", argv[currentarg+1]);
+					printf(" Valid parameters:\n");
+					printf("    0 - show packets per second (default)\n");
+					printf("    1 - show transfer counters\n");
+					return 1;
+				}
+				currentarg++;
+			}
 			livetraffic=1;
 			query=0;
 		} else if (strcmp(argv[currentarg],"--force")==0) {
@@ -228,6 +327,9 @@ int main(int argc, char *argv[]) {
 		} else if (strcmp(argv[currentarg],"--showconfig")==0) {
 			printcfgfile();
 			return 0;			
+		} else if (strcmp(argv[currentarg],"--delete")==0) {
+			delete=1;
+			query=0;
 		} else if (strcmp(argv[currentarg],"--iflist")==0) {
 			getiflist(&ifacelist);
 			printf("Available interfaces: %s\n", ifacelist);
@@ -274,14 +376,36 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
+	/* set used interface if none specified and make sure it's null terminated */
+	if (defaultiface) {
+		strncpy(interface, definterface, 32);
+	}
+	interface[31]='\0';
+
+	/* db merge */
+	if (query && strstr(interface, "+")) {
+		if (mergedb(interface, dirname)) {
+			files = merged = 1;
+		} else {
+			return 1;
+		}
+	}
+
+	/* save merged database */
+	if (merged && savemerged) {
+		data.lastupdated = 0;
+		if (writedb("mergeddb", ".", 2)) {
+			printf("Database saved as \"mergeddb\" in the current directory.\n");
+		}
+		return 0;
+	}
+
 	/* counter reset */
 	if (reset) {
 		if (!spacecheck(dirname) && !force) {
 			printf("Error: Not enough free diskspace available.\n");
 			return 1;
 		}
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		readdb(interface, dirname);
 		data.currx=0;
 		data.curtx=0;
@@ -296,8 +420,6 @@ int main(int argc, char *argv[]) {
 			printf("Error: Not enough free diskspace available.\n");
 			return 1;
 		}
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		if (!synccounters(interface, dirname)) {
 			return 1;
 		}
@@ -305,14 +427,35 @@ int main(int argc, char *argv[]) {
 			printf("Counters synced for \"%s\"\n", data.interface);
 	}
 
+	/* delete */
+	if (delete) {
+		if (force) {
+			if (checkdb(interface, dirname)) {
+				if (removedb(interface, dirname)) {
+					printf("Database for interface \"%s\" deleted.\n", interface);
+					printf("The interface will no longer be monitored.\n");
+					return 0;
+				} else {
+					printf("Error: Deleting database for interface \"%s\" failed.\n", interface);
+					return 1;
+				}
+			} else {
+					printf("Error: No database found for interface \"%s\".\n", interface);
+					return 1;				
+			}
+		} else {
+			printf("Warning:\nThe current option would delete the database for \"%s\".\n", interface);
+			printf("Use --force in order to really do that.\n");
+			return 1;
+		}		
+	}
+
 	/* clear top10 */
 	if (cleartop) {
 		if (!spacecheck(dirname) && !force) {
 			printf("Error: Not enough free diskspace available.\n");
 			return 1;
 		}
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		if (force) {
 			readdb(interface, dirname);
 
@@ -327,7 +470,7 @@ int main(int argc, char *argv[]) {
 			query=0;
 		} else {
 			printf("Warning:\nThe current option would clear the top10 for \"%s\".\n", interface);
-			printf("Use --force to override this message.\n");
+			printf("Use --force in order to really do that.\n");
 			return 1;
 		}
 	}
@@ -338,8 +481,6 @@ int main(int argc, char *argv[]) {
 			printf("Error: Not enough free diskspace available.\n");
 			return 1;
 		}
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		if (force) {
 			readdb(interface, dirname);
 
@@ -356,7 +497,7 @@ int main(int argc, char *argv[]) {
 			query=0;
 		} else {
 			printf("Warning:\nThe current option would rebuild total tranfers for \"%s\".\n", interface);
-			printf("Use --force to override this message.\n");
+			printf("Use --force in order to really do that.\n");
 			return 1;
 		}
 	}
@@ -367,8 +508,6 @@ int main(int argc, char *argv[]) {
 			printf("Error: Not enough free diskspace available.\n");
 			return 1;
 		}
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		newdb=readdb(interface, dirname);
 		if (!data.active && !newdb) {
 			data.active=1;
@@ -383,8 +522,6 @@ int main(int argc, char *argv[]) {
 			printf("Error: Not enough free diskspace available.\n");
 			return 1;
 		}
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		newdb=readdb(interface, dirname);
 		if (data.active && !newdb) {
 			data.active=0;
@@ -406,7 +543,7 @@ int main(int argc, char *argv[]) {
 		}
 		
 		/* update every file if -i isn't specified */
-		if (strcmp(interface, "default")==0) {
+		if (defaultiface) {
 			
 			dir=opendir(dirname);
 
@@ -457,14 +594,11 @@ int main(int argc, char *argv[]) {
 				update=0;
 			}
 			
-			/* reset to default */
-			strncpy(interface, "default", 32);
-		
 		/* update only selected file */
 		} else {
 			newdb=readdb(interface, dirname);
 			if (data.active) {
-				if (!getifinfo(data.interface)) {
+				if (!getifinfo(data.interface) && !force) {
 					getiflist(&ifacelist);
 					printf("Error: Interface \"%s\" couldn't be found.\n Only available interfaces can be added for monitoring.\n", data.interface);
 					printf("\n The following interfaces are currently available:\n    %s\n", ifacelist);
@@ -501,7 +635,7 @@ int main(int argc, char *argv[]) {
 	if (query) {
 	
 		/* show all interfaces if -i isn't specified */
-		if (strcmp(interface, "default")==0) {
+		if (defaultiface) {
 			
 			if (files==0) {
 				/* printf("No database found.\n"); */
@@ -509,7 +643,11 @@ int main(int argc, char *argv[]) {
 			} else if ((cfg.qmode==0 || cfg.qmode==8) && (files>1)) {
 
 				if (cfg.qmode==0) {
-					printf("\n                      rx      /      tx      /     total    /   estimated\n");
+					if (cfg.ostyle!=0) {
+						printf("\n                      rx      /      tx      /     total    /   estimated\n");
+					} else {
+						printf("\n                      rx      /      tx      /     total\n");
+					}
 				} else {
 					printf("<vnstat version=\"%s\" xmlversion=\"%d\">\n", VNSTATVERSION, XMLVERSION);
 				}
@@ -536,10 +674,17 @@ int main(int argc, char *argv[]) {
 				
 			/* show in qmode if there's only one file or qmode!=0 */
 			} else {
-				newdb=readdb(definterface, dirname);
+				if (!merged) {
+					newdb=readdb(definterface, dirname);
+				}
 				if (!newdb) {
-					if (cfg.qmode==5)
-						printf("\n                     rx      /     tx      /    total    /  estimated\n");
+					if (cfg.qmode==5) {
+						if (cfg.ostyle!=0) {
+							printf("\n                      rx      /      tx      /     total    /   estimated\n");
+						} else {
+							printf("\n                      rx      /      tx      /     total\n");
+						}
+					}
 					if (cfg.qmode!=8) {
 						showdb(cfg.qmode);
 					} else {
@@ -552,10 +697,17 @@ int main(int argc, char *argv[]) {
 		
 		/* show only specified file */
 		} else {
-			newdb=readdb(interface, dirname);
+			if (!merged) {
+				newdb=readdb(interface, dirname);
+			}
 			if (!newdb) {
-				if (cfg.qmode==5)
-					printf("\n                     rx      /     tx      /    total    /  estimated\n");
+				if (cfg.qmode==5) {
+					if (cfg.ostyle!=0) {
+						printf("\n                      rx      /      tx      /     total    /   estimated\n");
+					} else {
+						printf("\n                      rx      /      tx      /     total\n");
+					}
+				}
 				if (cfg.qmode!=8) {
 					showdb(cfg.qmode);
 				} else {
@@ -569,17 +721,12 @@ int main(int argc, char *argv[]) {
 	
 	/* calculate traffic */
 	if (traffic) {
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
 		trafficmeter(interface, cfg.sampletime);
 	}
 
 	/* live traffic */
 	if (livetraffic) {
-		if (strcmp(interface, "default")==0)
-			strncpy(interface, definterface, 32);
-
-		livetrafficmeter(interface);
+		livetrafficmeter(interface, livemode);
 	}
 	
 	/* if nothing was shown previously */
@@ -598,12 +745,15 @@ int main(int argc, char *argv[]) {
 			printf("Nothing to do. Use --help for help.\n");
 		}
 	}
+
+	/* cleanup */
+	ibwflush();
 	
 	return 0;
 }
 
 
-int synccounters(char iface[32], char dirname[512])
+int synccounters(const char *iface, const char *dirname)
 {
 	readdb(iface, dirname);
 	if (!getifinfo(iface)) {
