@@ -26,7 +26,7 @@ vnStat daemon - Copyright (c) 2008-09 Teemu Toivola <tst@iki.fi>
 int main(int argc, char *argv[])
 {
 	int currentarg, running = 1, updateinterval, dbcount, dodbsave, rundaemon;
-	int dbsaved = 1, showhelp = 1;
+	int dbsaved = 1, showhelp = 1, sync = 0, saveinterval, forcesave = 0;
 	uint32_t dbhash = 0;
 	char cfgfile[512], dirname[512];
 	DIR *dir;
@@ -39,7 +39,6 @@ int main(int argc, char *argv[])
 	rundaemon = 0;     /* daemon disabled by default */
 	cfgfile[0] = '\0';
 	prevdbupdate = prevdbsave = dbcount = 0;
-	updateinterval = 20;
 
 	/* early check for debug and config parameter */
 	if (argc > 1) {
@@ -49,6 +48,7 @@ int main(int argc, char *argv[])
 			} else if (strcmp(argv[currentarg],"--config")==0) {
 				if (currentarg+1<argc) {
 					strncpy(cfgfile, argv[currentarg+1], 512);
+					cfgfile[511] = '\0';
 					if (debug)
 						printf("Used config file: %s\n", cfgfile);
 					currentarg++;
@@ -66,11 +66,10 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	setlocale(LC_ALL, cfg.locale);
-
 	/* init dirname and other config settings */
 	strncpy(dirname, cfg.dbdir, 512);
 	updateinterval = cfg.updateinterval;
+	saveinterval = cfg.saveinterval*60;
 
 	/* parse parameters, maybe not the best way but... */
 	for (currentarg=1; currentarg<argc; currentarg++) {
@@ -79,7 +78,7 @@ int main(int argc, char *argv[])
 		if ((strcmp(argv[currentarg],"-?")==0) || (strcmp(argv[currentarg],"--help")==0)) {
 			break;
 		} else if (strcmp(argv[currentarg],"--config")==0) {
-			/* config has already been parsed earlier so not but to do here */
+			/* config has already been parsed earlier so nothing to do here */
 			currentarg++;
 			continue;
 		} else if ((strcmp(argv[currentarg],"-D")==0) || (strcmp(argv[currentarg],"--debug")==0)) {
@@ -89,12 +88,15 @@ int main(int argc, char *argv[])
 			showhelp = 0;
 		} else if ((strcmp(argv[currentarg],"-n")==0) || (strcmp(argv[currentarg],"--nodaemon")==0)) {
 			showhelp = 0;
+		} else if ((strcmp(argv[currentarg],"-s")==0) || (strcmp(argv[currentarg],"--sync")==0)) {
+			sync = 1;
 		} else if ((strcmp(argv[currentarg],"-v")==0) || (strcmp(argv[currentarg],"--version")==0)) {
 			printf("vnStat daemon %s by Teemu Toivola <tst at iki dot fi>\n", VNSTATVERSION);
 			return 0;
 		} else if ((strcmp(argv[currentarg],"-p")==0) || (strcmp(argv[currentarg],"--pidfile")==0)) {
 			if (currentarg+1<argc) {
 				strncpy(cfg.pidfile, argv[currentarg+1], 512);
+				cfg.pidfile[511] = '\0';
 				if (debug)
 					printf("Used pid file: %s\n", cfg.pidfile);
 				currentarg++;
@@ -114,6 +116,7 @@ int main(int argc, char *argv[])
 		printf(" vnStat daemon %s by Teemu Toivola <tst at iki dot fi>\n\n", VNSTATVERSION);
 		printf("         -d, --daemon         fork process to background\n");
 		printf("         -n, --nodaemon       stay in foreground attached to the terminal\n");
+		printf("         -s, --sync           sync interface counters on first update\n");
 		printf("         -D, --debug          show additional debug and disable daemon\n");
 		printf("         -?, --help           show this help\n");
 		printf("         -v, --version        show version\n");
@@ -141,6 +144,9 @@ int main(int argc, char *argv[])
 		if (dbcount==0) {
 			printf("Zero database found, exiting.\n");
 			return 1;
+		} else {
+			/* set counter back to zero so that dbs will be cached later */
+			dbcount = 0;
 		}
 	}
 
@@ -173,7 +179,7 @@ int main(int argc, char *argv[])
 
 		/* track interface status only if at least one database exists */
 		if (dbcount!=0) {
-			dbhash = dbcheck(dbhash);
+			dbhash = dbcheck(dbhash, &forcesave);
 		}
 
 		if ((current-prevdbupdate)>=updateinterval) {
@@ -181,22 +187,30 @@ int main(int argc, char *argv[])
 			updateinterval = cfg.updateinterval;
 
 			if (debug) {
-				datashow();
+				cacheshow();
 			}
 
 			/* read database list if cache is empty */
-			if (datacount()==0) {
+			if (dbcount==0) {
 		
 				if ((dir=opendir(dirname))!=NULL) {
-					dbcount = 0;
+
 					while ((di=readdir(dir))) {
 						if (di->d_name[0]!='.') {
-							if (debug)
+							if (debug) {
 								printf("\nProcessing file \"%s/%s\"...\n", dirname, di->d_name);
+							}
 							
-							if (!dataadd(di->d_name)) {
+							if (!cacheadd(di->d_name, sync)) {
 								snprintf(errorstring, 512, "Cache memory allocation failed, exiting.");
 								printe(PT_Error);
+
+								/* clean daemon stuff before exit */
+								if (rundaemon && !debug) {
+									close(pidfile);
+									unlink(cfg.pidfile);
+								}
+								ibwflush();
 								return 1;
 							}
 							
@@ -204,21 +218,31 @@ int main(int argc, char *argv[])
 						}
 					}
 
+					closedir(dir);
+					sync = 0;
+
 					/* disable update interval check for one loop if database list was refreshed */
-					/* otherwise rise default update interval since there's nothing else to do */
+					/* otherwise increase default update interval since there's nothing else to do */
 					if (dbcount) {
 						updateinterval = 0;
 						intsignal = 42;
 						prevdbsave = current;
+						/* list monitored interfaces to log */
+						cachestatus();
 					} else {
 						updateinterval = 120;
 					}
-
-					closedir(dir);
 				
 				} else {
 					snprintf(errorstring, 512, "Unable to access database directory \"%s\", exiting.", dirname);
 					printe(PT_Error);
+
+					/* clean daemon stuff before exit */
+					if (rundaemon && !debug) {
+						close(pidfile);
+						unlink(cfg.pidfile);
+					}
+					ibwflush();
 					return 1;
 				}
 
@@ -226,9 +250,17 @@ int main(int argc, char *argv[])
 			} else {
 				prevdbupdate = current;
 				datalist = dataptr;
-			
-				if ((current-prevdbsave)>=(cfg.saveinterval*60)) {
+
+				/* alter save interval if all interfaces are unavailable */
+				if (cacheactivecount()) {
+					saveinterval = cfg.saveinterval*60;
+				} else {
+					saveinterval = cfg.offsaveinterval*60;
+				}
+
+				if ((current-prevdbsave)>=(saveinterval) || forcesave) {
 					dodbsave = 1;
+					forcesave = 0;
 					prevdbsave = current;
 				} else {
 					dodbsave = 0;
@@ -242,7 +274,7 @@ int main(int argc, char *argv[])
 					}
 
 					/* get data from cache if available */
-					if (dataget(datalist->data.interface)==0) {
+					if (cacheget(datalist)==0) {
 						
 						/* try to read data from file if not cached */
 						if (readdb(datalist->data.interface, dirname)!=-1) {
@@ -258,7 +290,13 @@ int main(int argc, char *argv[])
 					/* get info if interface has been marked as active */
 					if (data.active) {
 						if (getifinfo(data.interface)) {
-							parseifinfo(0);
+							if (datalist->sync) { /* if --sync was used during startup */
+								data.currx = ifinfo.rx;
+								data.curtx = ifinfo.tx;
+								datalist->sync = 0;
+							} else {
+								parseifinfo(0);
+							}
 						} else {
 							/* disable interface since we can't access its data */
 							data.active = 0;
@@ -272,13 +310,20 @@ int main(int argc, char *argv[])
 					/* check that the time is correct */
 					if (current>=data.lastupdated) {
 						data.lastupdated = current;
-						dataupdate();
+						cacheupdate();
 					} else {
 						/* skip update if previous update is less than a day in the future */
 						/* otherwise exit with error message since the clock is problably messed */
 						if (data.lastupdated>(current+86400)) {
 							snprintf(errorstring, 512, "Interface \"%s\" has previous update date too much in the future, exiting.", data.interface);
 							printe(PT_Error);
+							
+							/* clean daemon stuff before exit */
+							if (rundaemon && !debug) {
+								close(pidfile);
+								unlink(cfg.pidfile);
+							}
+							ibwflush();
 							return 1;
 						} else {
 							datalist = datalist->next;
@@ -288,21 +333,37 @@ int main(int argc, char *argv[])
 
 					/* write data to file if now is the time for it */
 					if (dodbsave) {
-						if (spacecheck(dirname)) {
-							if (writedb(datalist->data.interface, dirname, 0)) {
-								if (!dbsaved) {
-									snprintf(errorstring, 512, "Database write possible again.");
-									printe(PT_Info);
-									dbsaved = 1;
+						if (checkdb(datalist->data.interface, dirname)) {
+							if (spacecheck(dirname)) {
+								if (writedb(datalist->data.interface, dirname, 0)) {
+									if (!dbsaved) {
+										snprintf(errorstring, 512, "Database write possible again.");
+										printe(PT_Info);
+										dbsaved = 1;
+									}
+								} else {
+									if (dbsaved) {
+										snprintf(errorstring, 512, "Unable to write database, continuing with cached data.");
+										printe(PT_Error);
+										dbsaved = 0;
+									}
+								}
+							} else {
+								/* show freespace error only once */
+								if (dbsaved) {
+									snprintf(errorstring, 512, "Free diskspace check failed, unable to write database, continuing with cached data.");
+									printe(PT_Error);
+									dbsaved = 0;
 								}
 							}
 						} else {
-							/* show freespace error only once */
-							if (dbsaved) {
-								snprintf(errorstring, 512, "Unable to write database, continuing with cached data.");
-								printe(PT_Error);
-								dbsaved = 0;
-							}
+							/* remove interface from update list since the database file doesn't exist anymore */
+							snprintf(errorstring, 512, "Database for interface \"%s\" no longer exists, removing from update list.", datalist->data.interface);
+							printe(PT_Info);
+							datalist = cacheremove(datalist->data.interface);
+							dbcount--;
+							cachestatus();
+							continue;
 						}
 					}
 		
@@ -326,9 +387,9 @@ int main(int argc, char *argv[])
 				case SIGHUP:
 					snprintf(errorstring, 512, "SIGHUP received, flushing data to disk and reloading config.");
 					printe(PT_Info);
-					dataflush(dirname);
+					cacheflush(dirname);
+					ibwflush();
 					if (loadcfg(cfgfile)) {
-						setlocale(LC_ALL, cfg.locale);
 						strncpy(dirname, cfg.dbdir, 512);
 					}
 					break;
@@ -359,7 +420,8 @@ int main(int argc, char *argv[])
 
 	} /* while */
 
-	dataflush(dirname);
+	cacheflush(dirname);
+	ibwflush();
 
 	/* clean daemon stuff */
 	if (rundaemon && !debug) {
@@ -379,14 +441,14 @@ void daemonize(void)
 		return; /* already a daemon */
 	}
 	
-	i = fork();
+	i = (int)fork();
 	
 	if (i<0) { /* fork error */
 		perror("fork");
-		exit(1); 
+		exit(EXIT_FAILURE); 
 	}
 	if (i>0) { /* parent exits */
-		exit(0);
+		exit(EXIT_SUCCESS);
 	}
 	/* child (daemon) continues */
 
@@ -396,7 +458,7 @@ void daemonize(void)
 		snprintf(errorstring, 512, "vnStat daemon %s started.", VNSTATVERSION);
 		if (!printe(PT_Info)) {
 			printf("Error: Unable to use logfile. Exiting.\n");
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -406,13 +468,13 @@ void daemonize(void)
 		perror("pidfile");
 		snprintf(errorstring, 512, "pidfile failed, exiting.");
 		printe(PT_Error);
-		exit(1); /* can't open */
+		exit(EXIT_FAILURE); /* can't open */
 	}
 	if (lockf(pidfile,F_TLOCK,0)<0) {
 		perror("pidfile lock");
 		snprintf(errorstring, 512, "pidfile lock failed, exiting.");
 		printe(PT_Error);
-		exit(1); /* can't lock */
+		exit(EXIT_FAILURE); /* can't lock */
 	}
 
 	/* close all descriptors except lock file */
@@ -424,15 +486,42 @@ void daemonize(void)
 
 	/* redirect standard i/o to null */
 	i=open("/dev/null",O_RDWR); /* stdin */
-	dup(i); /* stdout */
-	dup(i); /* stderr */
+
+	/* stdout */
+	if (dup(i) < 0) {
+		perror("dup(stdout)");
+		snprintf(errorstring, 512, "dup(stdout) failed, exiting.");
+		printe(PT_Error);
+		exit(EXIT_FAILURE);
+	}
+	/* stderr */
+	if (dup(i) < 0) {
+		perror("dup(stderr)");
+		snprintf(errorstring, 512, "dup(stderr) failed, exiting.");
+		printe(PT_Error);
+		exit(EXIT_FAILURE);
+	}
 	
 	umask(027); /* set newly created file permissions */
-	chdir("/"); /* change running directory */
+	
+	/* change running directory */
+	if (chdir("/") < 0) {
+		perror("chdir(/)");
+		snprintf(errorstring, 512, "directory change to / failed, exiting.");
+		printe(PT_Error);
+		exit(EXIT_FAILURE);
+	}
 
 	/* first instance continues */
-	sprintf(str,"%d\n",getpid());
-	write(pidfile,str,strlen(str)); /* record pid to lockfile */
+	snprintf(str, 10, "%d\n", (int)getpid());
+
+	/* record pid to lockfile */
+	if (write(pidfile,str,strlen(str)) < 0) {
+		perror("write(pidfile)");
+		snprintf(errorstring, 512, "writing to pidfile %s failed, exiting.", cfg.pidfile);
+		printe(PT_Error);
+		exit(EXIT_FAILURE);
+	}
 
 	signal(SIGCHLD,SIG_IGN); /* ignore child */
 	signal(SIGTSTP,SIG_IGN); /* ignore tty signals */
@@ -440,7 +529,7 @@ void daemonize(void)
 	signal(SIGTTIN,SIG_IGN);
 
 	if (cfg.uselogging==1) {
-		snprintf(errorstring, 512, "Daemon running with pid %d.", getpid());
+		snprintf(errorstring, 512, "Daemon running with pid %d.", (int)getpid());
 		printe(PT_Info);
 	}
 }
