@@ -19,12 +19,13 @@ vnStat image output - Copyright (c) 2007-09 Teemu Toivola <tst@iki.fi>
 #include "image.h"
 #include "cfg.h"
 #include "dbaccess.h"
+#include "dbmerge.h"
 #include "vnstati.h"
 
 int main(int argc, char *argv[])
 {
 	FILE *pngout;
-	int currentarg, cache=0, help=0, showheader=1, showedge=1, showrate=0;
+	int currentarg, cache=0, help=0, showheader=1, showedge=1;
 	char interface[32], dirname[512], filename[512], cfgfile[512];
 	struct stat filestat;
 
@@ -40,6 +41,7 @@ int main(int argc, char *argv[])
 			} else if (strcmp(argv[currentarg],"--config")==0) {
 				if (currentarg+1<argc) {
 					strncpy(cfgfile, argv[currentarg+1], 512);
+					cfgfile[511] = '\0';
 					if (debug)
 						printf("Used config file: %s\n", cfgfile);
 					currentarg++;
@@ -58,7 +60,13 @@ int main(int argc, char *argv[])
 	}
 	cfg.qmode = 0;
 
-	setlocale(LC_ALL, cfg.locale);
+	if (cfg.locale[0]!='-' && strlen(cfg.locale)>0) {
+		setlocale(LC_ALL, cfg.locale);
+	} else {
+		if (getenv("LC_ALL")) {
+			setlocale(LC_ALL, getenv("LC_ALL"));
+		}
+	}
 	strncpy(interface, cfg.iface, 32);
 	strncpy(dirname, cfg.dbdir, 512);
 	filename[0] = '\0';
@@ -73,8 +81,9 @@ int main(int argc, char *argv[])
 		} else if ((strcmp(argv[currentarg],"-i")==0) || (strcmp(argv[currentarg],"--iface"))==0) {
 			if (currentarg+1<argc) {
 				strncpy(interface, argv[currentarg+1], 32);
+				interface[31] = '\0';
 				if (debug)
-					printf("Used interface: \"%s\"\n", filename);
+					printf("Used interface: \"%s\"\n", interface);
 				currentarg++;
 				continue;
 			} else {
@@ -84,6 +93,7 @@ int main(int argc, char *argv[])
 		} else if ((strcmp(argv[currentarg],"-o")==0) || (strcmp(argv[currentarg],"--output"))==0) {
 			if (currentarg+1<argc) {
 				strncpy(filename, argv[currentarg+1], 512);
+				filename[511] = '\0';
 				if (debug)
 					printf("Output file: \"%s\"\n", filename);
 				currentarg++;
@@ -103,8 +113,62 @@ int main(int argc, char *argv[])
 				printf("Error: Parameter for -c missing or invalid.\n");
 				return 1;
 			}
+		} else if ((strcmp(argv[currentarg],"--style"))==0) {
+			if (currentarg+1<argc && isdigit(argv[currentarg+1][0])) {
+				cfg.ostyle = atoi(argv[currentarg+1]);
+				if (cfg.ostyle > 3 || cfg.ostyle < 0) {
+					printf("Error: Invalid style parameter \"%d\" for --style.\n", cfg.ostyle);
+					return 1;
+				}
+				if (debug)
+					printf("Style changed: %d\n", cfg.ostyle);
+				currentarg++;
+				continue;
+			} else {
+				printf("Error: Style parameter for --style missing.\n");
+				return 1;
+			}
+		} else if ((strcmp(argv[currentarg],"--transparent"))==0) {
+			if (currentarg+1<argc && isdigit(argv[currentarg+1][0])) {
+				cfg.transbg = atoi(argv[currentarg+1]);
+				if (cfg.transbg > 1 || cfg.transbg < 0) {
+					printf("Error: Invalid parameter \"%d\" for --transparent.\n", cfg.transbg);
+					return 1;
+				}
+				if (debug)
+					printf("Transparency changed: %d\n", cfg.transbg);
+				currentarg++;
+				continue;
+			} else {
+				cfg.transbg = !cfg.transbg;
+				if (debug)
+					printf("Transparency changed: %d\n", cfg.transbg);
+			}
+		} else if ((strcmp(argv[currentarg],"--dbdir"))==0) {
+			if (currentarg+1<argc) {
+				strncpy(dirname, argv[currentarg+1], 512);
+				dirname[511] = '\0';
+				if (debug)
+					printf("DatabaseDir: \"%s\"\n", dirname);
+				currentarg++;
+				continue;
+			} else {
+				printf("Error: Directory for --dbdir missing.\n");
+				return 1;
+			}
+		} else if ((strcmp(argv[currentarg],"--locale"))==0) {
+			if (currentarg+1<argc) {
+				setlocale(LC_ALL, argv[currentarg+1]);
+				if (debug)
+					printf("Locale: \"%s\"\n", argv[currentarg+1]);
+				currentarg++;
+				continue;
+			} else {
+				printf("Error: Locale for --locale missing.\n");
+				return 1;
+			}
 		} else if (strcmp(argv[currentarg],"--config")==0) {
-			/* config has already been parsed earlier so not but to do here*/
+			/* config has already been parsed earlier so nothing to do here */
 			currentarg++;
 			continue;				
 		} else if ((strcmp(argv[currentarg],"-D")==0) || (strcmp(argv[currentarg],"--debug"))==0) {
@@ -127,8 +191,25 @@ int main(int argc, char *argv[])
 			showheader = 0;
 		} else if ((strcmp(argv[currentarg],"-ne")==0) || (strcmp(argv[currentarg],"--noedge"))==0) {
 			showedge = 0;
-		} else if ((strcmp(argv[currentarg],"-r")==0) || (strcmp(argv[currentarg],"--rate"))==0) {
-			showrate = 1;
+		} else if ((strcmp(argv[currentarg],"-ru")==0) || (strcmp(argv[currentarg],"--rateunit"))==0) {
+			if (currentarg+1<argc && isdigit(argv[currentarg+1][0])) {
+				cfg.rateunit = atoi(argv[currentarg+1]);
+				if (cfg.rateunit > 1 || cfg.rateunit < 0) {
+					printf("Error: Invalid parameter \"%d\" for --rateunit.\n", cfg.rateunit);
+					printf(" Valid parameters:\n");
+					printf("    0 - bytes\n");
+					printf("    1 - bits\n");
+					return 1;
+				}
+				if (debug)
+					printf("Rateunit changed: %d\n", cfg.rateunit);
+				currentarg++;
+				continue;
+			} else {
+				cfg.rateunit = !cfg.rateunit;
+				if (debug)
+					printf("Rateunit changed: %d\n", cfg.rateunit);
+			}
 		} else if ((strcmp(argv[currentarg],"-v")==0) || (strcmp(argv[currentarg],"--version"))==0) {
 			printf("vnStat image output %s by Teemu Toivola <tst at iki dot fi>\n", VNSTATVERSION);
 			return 0;
@@ -140,23 +221,27 @@ int main(int argc, char *argv[])
 
 	if (help || argc == 1) {
 		printf(" vnStat image output %s by Teemu Toivola <tst at iki dot fi>\n\n", VNSTATVERSION);
-		printf("         -h,  --hours             output hours\n");
-		printf("         -d,  --days              output days\n");
-		printf("         -m,  --months            output months\n");
-		printf("         -t,  --top10             output top10\n");
-		printf("         -s,  --summary           output summary\n");
-		printf("         -hs, --hsummary          output horizontal summary with hours\n");
-		printf("         -vs, --vsummary          output vertical summary with hours\n");
-		printf("         -nh, --noheader          remove header from output\n");
-		printf("         -ne, --noedge            remove edge from output\n");
-		printf("         -r,  --rate              show hours with rate\n");
-		printf("         -o,  --output            select output filename\n");
-		printf("         -c,  --cache             update output only when too old\n");
-		printf("         -i,  --iface             used interface (default: %s)\n", interface);
-		printf("         -?,  --help              this help\n");
-		printf("         -D,  --debug             show some additional debug information\n");
-		printf("         -v,  --version           show version\n");
-		printf("         --config                 select used config file\n\n");
+		printf("         -h,  --hours          output hours\n");
+		printf("         -d,  --days           output days\n");
+		printf("         -m,  --months         output months\n");
+		printf("         -t,  --top10          output top10\n");
+		printf("         -s,  --summary        output summary\n");
+		printf("         -hs, --hsummary       output horizontal summary with hours\n");
+		printf("         -vs, --vsummary       output vertical summary with hours\n");
+		printf("         -nh, --noheader       remove header from output\n");
+		printf("         -ne, --noedge         remove edge from output\n");
+		printf("         -ru, --rateunit       swap configured rate unit\n");
+		printf("         -o,  --output         select output filename\n");
+		printf("         -c,  --cache          update output only when too old\n");
+		printf("         -i,  --iface          used interface (default: %s)\n", interface);
+		printf("         -?,  --help           this help\n");
+		printf("         -D,  --debug          show some additional debug information\n");
+		printf("         -v,  --version        show version\n");
+		printf("         --dbdir               select database directory\n");
+		printf("         --style               select output style (0-3)\n");
+		printf("         --locale              set locale\n");
+		printf("         --config              select config file\n");
+		printf("         --transparent         toggle background transparency\n\n");
 		printf("See also \"man vnstati\".\n");
 		return 0;
 	}
@@ -185,9 +270,15 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	/* load database */
-	if (readdb(interface, dirname)==1) {
-		return 1;
+	/* load database and do merge if needed */
+	if (strstr(interface, "+")) {
+		if (!mergedb(interface, dirname)) {
+			return 1;
+		}
+	} else {
+		if (readdb(interface, dirname)==1) {
+			return 1;
+		}
 	}
 
 	/* open file */	
@@ -219,26 +310,46 @@ int main(int argc, char *argv[])
 			drawtop(showheader, showedge);
 			break;
 		case 5:
-			drawsummary(0, showheader, showedge, showrate);
+			if (cfg.slayout) {
+				drawsummary(0, showheader, showedge, 0);
+			} else {
+				drawoldsummary(0, showheader, showedge, 0);
+			}
 			break;
 		case 51:
-			drawsummary(1, showheader, showedge, showrate);
+			if (cfg.slayout) {
+				drawsummary(1, showheader, showedge, cfg.hourlyrate);
+			} else {
+				drawoldsummary(1, showheader, showedge, cfg.hourlyrate);
+			}
 			break;
 		case 52:
-			drawsummary(2, showheader, showedge, showrate);
+			if (cfg.slayout) {
+				drawsummary(2, showheader, showedge, cfg.hourlyrate);
+			} else {
+				drawoldsummary(2, showheader, showedge, cfg.hourlyrate);
+			}
 			break;
 		case 7:
-			drawhourly(showheader, showedge, showrate);
+			drawhourly(showheader, showedge, cfg.hourlyrate);
 			break;
 		default:
 			break;
 	}
 
+	/* enable background transparency if needed */
+	if (cfg.transbg) {
+		gdImageColorTransparent(im, cbackground);
+	}
+
 	/* write image */
 	gdImagePng(im, pngout);
 	fclose(pngout);
 	gdImageDestroy(im);
 
+	/* cleanup */
+	ibwflush();
+
 	if (debug)
 		printf("all done\n");
 
